@@ -1,9 +1,5 @@
-### R code from vignette source 'vignette.Rnw'
-### Encoding: UTF-8
-
-###################################################
-### code chunk number 1: vignette.Rnw:82-119
-###################################################
+## ----echo=FALSE----------------------------------------------------------
+library(coloc)
 setClass("simdata",
          representation(df1="data.frame",df2="data.frame"))
 setValidity("simdata", function(object) {
@@ -15,7 +11,7 @@ setMethod("show", signature="simdata", function(object) {
   cat("pair of simulated datasets, with",ncol(object@df1)-1,"SNPs and",nrow(object@df1),"samples.\n")
 })
 
-sim.data <- function(nsnps=10,nsamples=200,causals=1:2,nsim=1) {
+sim.data <- function(nsnps=50,nsamples=200,causals=1:2,nsim=1) {
   cat("Generate",nsim,"small sets of data\n")
   ntotal <- nsnps * nsamples * nsim
   X1 <- matrix(rbinom(ntotal,1,0.4)+rbinom(ntotal,1,0.4),ncol=nsnps)
@@ -38,65 +34,47 @@ sim.data <- function(nsnps=10,nsamples=200,causals=1:2,nsim=1) {
 }
 
 ## simulate some data and load the coloc library
-set.seed(12345)
+set.seed(46411)
 data <- sim.data(nsamples=1000,nsim=1)
 library(coloc)
 
+## ------------------------------------------------------------------------
+if(requireNamespace("snpStats")) {
+library(snpStats)
+Y1 <- data@df1$Y
+X1 <- new("SnpMatrix",as.matrix(data@df1[,-1]))
+p1 <- snpStats::p.value(single.snp.tests(phenotype=Y1, snp.data=X1),df=1)
+maf <- col.summary(X1)[,"MAF"]
 
-###################################################
-### code chunk number 2: vignette.Rnw:133-138
-###################################################
-## run a coloc with pcs
-pcs <- pcs.prepare(data@df1[,-1], data@df2[,-1])
-pcs.1 <- pcs.model(pcs, group=1, Y=data@df1[,1], threshold=0.8)
-pcs.2 <- pcs.model(pcs, group=2, Y=data@df2[,1], threshold=0.8)
-ct.pcs <- coloc.test(pcs.1,pcs.2)
+get.beta <- function(...) {
+   tmp <- snpStats::snp.rhs.estimates(..., family="gaussian")
+   beta <- sapply(tmp,"[[","beta")
+   varbeta <- sapply(tmp, "[[", "Var.beta")
+   return(list(beta=beta,varbeta=varbeta))
+}
+b1 <- get.beta(Y1 ~ 1, snp.data=X1)
+} else {
+Y1 <- data@df1$Y
+X1 <- as.matrix(data@df1[,-1])
+tests1 <- lapply(1:ncol(X1), function(i) summary(lm(Y1 ~ X1[,i]))$coefficients[2,])
+p1 <- sapply(tests1,"[",4)
+maf <- colMeans(X2)/2
 
+get.beta <- function(x) {
+   beta <- sapply(x,"[",1)
+   varbeta <- sapply(x, "[", 2)^2
+   return(list(beta=beta,varbeta=varbeta))
+}
+b1 <- get.beta(tests1)
+}
 
-###################################################
-### code chunk number 3: vignette.Rnw:148-150
-###################################################
-ct.pcs
-str(summary(ct.pcs))
+## ------------------------------------------------------------------------
+my.res <- finemap.abf(dataset=list(beta=b1$beta, varbeta=b1$varbeta, N=nrow(X1),sdY=sd(Y1),type="quant"))
+head(my.res)
+tail(my.res)
 
-
-###################################################
-### code chunk number 4: vignette.Rnw:164-166
-###################################################
-ct.pcs.bayes <- coloc.test(pcs.1,pcs.2, bayes=TRUE)
-ci(ct.pcs.bayes)
-
-
-###################################################
-### code chunk number 5: vignette.Rnw:174-182
-###################################################
-ct.bma <- coloc.bma(data@df1, data@df2, 
-                    family1="gaussian", family2="gaussian",
-                    plot.coeff=TRUE)
-ct.bma.bayes <- coloc.bma(data@df1, data@df2, 
-                          family1="gaussian", family2="gaussian", 
-                          bayes=TRUE)
-ct.bma
-ci(ct.bma.bayes)
-
-
-###################################################
-### code chunk number 6: vignette.Rnw:224-233
-###################################################
-## compare individual values of eta
-ct.pcs <- coloc.test(pcs.1,pcs.2, bayes.factor=c(-1,0,1))
-bf(ct.pcs)
-
-## compare ranges of eta
-ct.bma <- coloc.bma(data@df1, data@df2, 
-                    family1="gaussian", family2="gaussian",
-                    bayes.factor=list(c(-0.1,1), c(0.9,1.1)))
-bf(ct.bma)
-
-
-###################################################
-### code chunk number 7: vignette.Rnw:283-294
-###################################################
+## ------------------------------------------------------------------------
+if(requireNamespace("snpStats")) {
 library(snpStats)
 
 Y1 <- data@df1$Y
@@ -107,22 +85,86 @@ X2 <- new("SnpMatrix",as.matrix(data@df2[,-1]))
 
 p1 <- snpStats::p.value(single.snp.tests(phenotype=Y1, snp.data=X1),df=1)
 p2 <- snpStats::p.value(single.snp.tests(phenotype=Y2, snp.data=X2),df=1)
+
 maf <- col.summary(X2)[,"MAF"]
 
+get.beta <- function(...) {
+   tmp <- snpStats::snp.rhs.estimates(..., family="gaussian")
+   beta <- sapply(tmp,"[[","beta")
+   varbeta <- sapply(tmp, "[[", "Var.beta")
+   return(list(beta=beta,varbeta=varbeta))
+}
+b1 <- get.beta(Y1 ~ 1, snp.data=X1)
+b2 <- get.beta(Y2 ~ 1, snp.data=X2)
+} else {
+Y1 <- data@df1$Y
+Y2 <- data@df2$Y
 
-###################################################
-### code chunk number 8: vignette.Rnw:301-305
-###################################################
+X1 <- as.matrix(data@df1[,-1])
+X2 <- as.matrix(data@df2[,-1])
+
+tests1 <- lapply(1:ncol(X1), function(i) summary(lm(Y1 ~ X1[,i]))$coefficients[2,])
+tests2 <- lapply(1:ncol(X2), function(i) summary(lm(Y2 ~ X2[,i]))$coefficients[2,])
+
+p1 <- sapply(tests1,"[",4)
+p2 <- sapply(tests2,"[",4)
+
+maf <- colMeans(X2)/2
+
+get.beta <- function(x) {
+   beta <- sapply(x,"[",1)
+   varbeta <- sapply(x, "[", 2)^2
+   return(list(beta=beta,varbeta=varbeta))
+}
+b1 <- get.beta(tests1)
+b2 <- get.beta(tests2)
+}
+
+## ------------------------------------------------------------------------
+my.res <- coloc.abf(dataset1=list(beta=b1$beta, varbeta=b1$varbeta, N=nrow(X1),sdY=sd(Y1),type="quant"),
+                    dataset2=list(beta=b2$beta, varbeta=b2$varbeta, N=nrow(X2),sdY=sd(Y2),type="quant"),
+                    MAF=maf)
+print(my.res[[1]])
+
+## ------------------------------------------------------------------------
+my.res <- coloc.abf(dataset1=list(beta=b1$beta, varbeta=b1$varbeta, N=nrow(X1),type="quant"),
+                    dataset2=list(beta=b2$beta, varbeta=b2$varbeta, N=nrow(X2),type="quant"),
+                    MAF=maf)
+print(my.res[[1]])
+
+## ------------------------------------------------------------------------
 my.res <- coloc.abf(dataset1=list(pvalues=p1,N=nrow(X1),type="quant"),
                     dataset2=list(pvalues=p2,N=nrow(X2),type="quant"),
                     MAF=maf)
 print(my.res[[1]])
 
-
-###################################################
-### code chunk number 9: vignette.Rnw:313-315
-###################################################
+## ------------------------------------------------------------------------
 ct.abf <- coloc.abf.datasets(data@df1, data@df2, response1="Y", response2="Y",
                              type1="quant", type2="quant")
 
+## ----fig=TRUE------------------------------------------------------------
+## run a coloc with pcs
+pcs <- pcs.prepare(data@df1[,-1], data@df2[,-1])
+pcs.1 <- pcs.model(pcs, group=1, Y=data@df1[,1], threshold=0.8)
+pcs.2 <- pcs.model(pcs, group=2, Y=data@df2[,1], threshold=0.8)
+ct.pcs <- coloc.test(pcs.1,pcs.2)
+
+## ------------------------------------------------------------------------
+ct.pcs
+str(summary(ct.pcs))
+
+## ------------------------------------------------------------------------
+ct.pcs.bayes <- coloc.test(pcs.1,pcs.2, bayes=TRUE)
+ci(ct.pcs.bayes)
+
+## ------------------------------------------------------------------------
+## compare individual values of eta
+ct.pcs <- coloc.test(pcs.1,pcs.2, bayes.factor=c(-1,0,1))
+bf(ct.pcs)
+
+## compare ranges of eta
+ct.bma <- coloc.bma(data@df1, data@df2, 
+                    family1="gaussian", family2="gaussian",
+                    bayes.factor=list(c(-0.1,1), c(0.9,1.1)))
+bf(ct.bma)
 
